/****************************************************************
 BackwardAlgorithm.C
 bmajoros@tigr.org

 Copyright (c)2003, The Institute for Genomic Research (TIGR),
 Rockville, Maryland, U.S.A.  All rights reserved.  This software
 is governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "BackwardAlgorithm.H"
#include <math.h>
#include <iostream>
#include "BOOM/String.H"


BackwardAlgorithm::BackwardAlgorithm(HiddenMarkovModel &hmm,Sequence &sequence)
  : scalingFactors(sequence.getLength()+1),
    hmm(hmm),
    dpMatrix(hmm.countStates(),sequence.getLength()+1),
    sequence(sequence),
    numStates(hmm.countStates()),
    seqLen(sequence.getLength())
{
  compute();
}



const Array1D<double> &BackwardAlgorithm::getScalingFactors()
{
  return scalingFactors;
}



double BackwardAlgorithm::getScaledP()
{
  return P;
}



double BackwardAlgorithm::operator()(int s,int i)
{
  return dpMatrix[s][i];
}



void BackwardAlgorithm::compute()
{
  for(int k=0 ; k<numStates ; ++k) 
    dpMatrix[k][seqLen]=hmm.getTransitionProb(k,0);
  for(int i=seqLen-1 ; i>=0 ; --i)
    for(int k=0 ; k<numStates ; ++k)
      {
	sum=0.0;	
	for(int l=1 ; l<numStates ; ++l)
	  sum+=dpMatrix[l][i+1] * hmm.getEmissionProb(l,sequence[i]) 
	    * hmm.getTransitionProb(k,l);
	double scalingValue=0.0;
	for(int kk=0 ; kk<numStates ; ++kk)
	  for(int l=1 ; l<numStates ; ++l)
	    scalingValue+=dpMatrix[l][i+1] * 
	      hmm.getEmissionProb(l,sequence[i]) 
	      * hmm.getTransitionProb(kk,l);
	if(isnan(scalingValue) || isinf(scalingValue))
	  throw String("Backard scaling value ")+(i+1)+"="+scalingValue;
	if(scalingValue<0 || scalingValue>10) //###
	  cerr << "Backward scaling value " << (i+1) << "=" 
	       << scalingValue << endl;
	scalingFactors[i+1]=scalingValue;
	double entry;
	if(scalingValue==0.0) entry=dpMatrix[k][i]=0.0;
	else entry=dpMatrix[k][i]=sum/scalingValue;
        if(isinf(entry)) throw NegativeInfinityException();
	if(isnan(entry) || isinf(entry) || entry<0 || entry>1)
	  throw String("b(")+k+","+i+")="+entry;
      }
  P=dpMatrix[0][0];
}



