/****************************************************************
 BaumWelch.H
 bmajoros@duke.edu

 This is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
 ****************************************************************/

#ifndef INCL_BaumWelch_H
#define INCL_BaumWelch_H

#include "ForwardAlgorithm.H"
#include "BackwardAlgorithm.H"
#include "HMMGraph.H"
#include <math.h>
#include <iostream>
#include <strstream>
#include <fstream>
#include <float.h>
#include "BOOM/Vector.H"
#include "BOOM/Array1D.H"
#include "BOOM/Sequence.H"
#include "BOOM/Progress.H"
using namespace std;


class BaumWelch
{
public:
  BaumWelch(HiddenMarkovModel &,long maxIterations,
	    Vector<Sequence*> &trainingSet);
private:
  Alphabet &alphabet;
  Progress progress;
  Array1D<double> likelihoods, scalingRatios;
  Array2D<double> A; // expected transition counts
  Array2D<double> E; // expected emission counts
  HiddenMarkovModel &hmm;
    HMMGraph hmmGraph;
  long maxIterations;
  double logLikelihoodThreshold, sum;
  Vector<Sequence*> &trainingSet;
  int numHmmStates, numTrain, alphabetSize;
  
  void reviseExpectedEmitCounts(int sequenceLength,
				ForwardAlgorithm &,BackwardAlgorithm &,
				Sequence &);
  void reviseExpectedTransCounts(int sequenceLength,ForwardAlgorithm &,
				 BackwardAlgorithm &,
				 Sequence &);
  void mainAlgorithm();
};


#endif
