/**************************************************************
FastForward.H
bmajoros@duke.edu

This is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
***************************************************************/
#ifndef INCL_FastForward_H
#define INCL_FastForward_H
#include "HMMGraph.H"
#include "BOOM/Array2D.H"
#include "BOOM/Array1D.H"
#include "HiddenMarkovModel.H"
#include "BOOM/Sequence.H"

class HiddenMarkovModel;

class FastForward
{
public:
  FastForward(HiddenMarkovModel &,HMMGraph &,Sequence &);
  FastForward(HiddenMarkovModel &,HMMGraph &,Sequence &,int begin,int len);
  double getLogP();
  const DblArray1D &getScalingFactors() const;
  bool PValueExceeds(FastForward &);
  double operator()(int state,int pos); // uses 1-based indices...
  double getScaledP();
private:
  double P;
  Sequence &sequence;
  int numStates, seqLen;
  DblArray1D scalingFactors;
  Array2D<double> dpMatrix;
  HiddenMarkovModel &hmm;
  HMMGraph &hmmGraph;
  const int offset;
  void computeDPMatrix();
  double computeScalingFactor(int);
};


#endif
