/**************************************************************
FastViterbi.C
bmajoros@duke.edu

This is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
***************************************************************/
#include <math.h>
#include <float.h>
#include <iostream>
#include "FastViterbi.H"
#include "BOOM/Constants.H"
using namespace std;

#define MOST_NEGATIVE -DBL_MAX

inline double safeAdd(double a,double b)
{
  if(a==MOST_NEGATIVE || b==MOST_NEGATIVE) return MOST_NEGATIVE;
  else return a+b;
}


FastViterbi::FastViterbi(HiddenMarkovModel &hmm)
  : numStates(hmm.countStates()), hmmGraph(hmm)
{
  // ctor
}



Vector<int> *FastViterbi::buildPath(int L,int finalState,
					Array2D<short> &linkBack)
{
  int currentState=finalState;
  Vector<int> &path=*new Vector<int>(L);
  path[L-1]=finalState;
  for(int pos=L-1 ; pos>0 ; --pos)
    {
      currentState=linkBack[currentState][pos+1];
      path[pos-1]=currentState;
    }
  return &path;
}



/*
  This method returns the state sequence of
  the most probable path (not including the
  start state).
 */
Vector<int> *FastViterbi::getPath(Sequence &sequence)
{
  HiddenMarkovModel &hmm=hmmGraph.getHMM();
  int L=sequence.getLength(), Lplus1=L+1;
  Array2D<short> linkBack(numStates,Lplus1);
  Array2D<double> m(numStates,Lplus1);
  m.setAllTo(NEGATIVE_INFINITY);
  m[0][0]=0.0;
  for(int i=1 ; i<=L ; ++i)
    {
      Symbol s=sequence[i-1];
      Vector<StateDoublePair> &statesEmittingS=
	hmmGraph.statesEmitting(s);
      int numEmitting=statesEmittingS.size();
      for(int l=0 ; l<numEmitting ; ++l)
	{
	  StateDoublePair &currentPair=statesEmittingS[l];
	  Vector<StateDoublePair> &precedingStates=
	    hmmGraph.statesPreceding(currentPair.state);
	  int numPreceding=precedingStates.size();
	  double bestP=log(0.0);
	  int bestPredecessor;
	  for(int j=0 ; j<numPreceding ; ++j)
	    {
	      StateDoublePair precedingPair=precedingStates[j];
	      double inductiveP=m[precedingPair.state][i-1];
	      double newP=safeAdd(inductiveP,precedingPair.logP);
	      if(newP>bestP)
		{
		  bestP=newP;
		  bestPredecessor=precedingPair.state;
		}
	    }
	  m[currentPair.state][i]=safeAdd(bestP,currentPair.logP);
	  linkBack[currentPair.state][i]=bestPredecessor;
	}
    }
  int bestState=1;
  double bestScore=NEGATIVE_INFINITY;
  for(int i=0 ; i<numStates ; ++i)
    if(m[i][L-1]>bestScore)
      {
	bestState=i;
	bestScore=m[i][L-1];
      }
  return buildPath(L,bestState,linkBack);
}



