/**************************************************************
HMMGraph.C
bmajoros@duke.edu

This is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
***************************************************************/

#include "HMMGraph.H"
#include <iostream>
#include <math.h>
using namespace std;


HMMGraph::HMMGraph(HiddenMarkovModel &hmm)
  : hmm(hmm), 
    predecessors(hmm.countStates()),
    successors(hmm.countStates()),
    emissions(hmm.getAlphabet().getNumElements())
{
  // ctor

  init();
}



void HMMGraph::init()
{
  int numStates=hmm.countStates();

  for(int i=0 ; i<numStates ; ++i)
    for(int j=0 ; j<numStates ; ++j)
      {
	double p=hmm.getTransitionProb(i,j);
	if(p>0)
	  {
	    predecessors[j].push_back(StateDoublePair(i,log(p),p));
	    successors[i].push_back(StateDoublePair(j,log(p),p));
	  }
      }

  Alphabet &alphabet=hmm.getAlphabet();
  int numSymbols=alphabet.getNumElements();
  for(int state=0 ; state<numStates ; ++state)
    for(int symbol=0 ; symbol<numSymbols ; ++symbol)
      {
	double p=hmm.getEmissionProb(state,symbol);
	if(p>0)
	  emissions[symbol].push_back(StateDoublePair(state,log(p),p));
      }
}



Vector<StateDoublePair> &HMMGraph::statesEmitting(Symbol s)
{
  return emissions[s];
}



Vector<StateDoublePair> &HMMGraph::statesPreceding(int state)
{
  return predecessors[state];
}



Vector<StateDoublePair> &HMMGraph::statesFollowing(int state)
{
  return successors[state];
}



HiddenMarkovModel &HMMGraph::getHMM()
{
  return hmm;
}

