/**************************************************************
HiddenMarkovModel.H
bmajoros@duke.edu

This is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
***************************************************************/
#ifndef INCL_HiddenMarkovModel_H
#define INCL_HiddenMarkovModel_H
#include <iostream>
#include "BOOM/String.H"
#include "BOOM/Sequence.H"
#include "BOOM/Array2D.H"
using namespace std;


class NegativeInfinityException {};


class HiddenMarkovModel
{
public:
  HiddenMarkovModel(const String &fname,Alphabet &);
  HiddenMarkovModel(istream &,Alphabet &);
  HiddenMarkovModel(Alphabet &,int numStates);
  HiddenMarkovModel(const HiddenMarkovModel &);
  double getEmissionProb(int inState,Symbol);
  void setTransitionProb(int fromState,int toState,double p);
  void setEmissionProb(int inState,Symbol,double p);
  bool doesTransitionExist(int from,int to);
  int countStates();
  Alphabet &getAlphabet();
  void printOn(ostream &);
  double getTransitionProb(int fromState,int toState);
  virtual bool save(const String &filename);
  virtual void load(const String &filename);
  virtual bool save(ostream &);
  virtual void load(istream &);
  void reverseComp();
  void normalizeTransitions();
  void normalizeEmissions();
  void addTransition(int from,int to);
  double getLogP(Sequence &seq);
  void randomizeEmissionProbs();
private:
  Array2D<double> transitionProb;
  int numStates;
  Array2D<double> emissionProb;
  Alphabet &alphabet;

  void swap(Symbol,Symbol);
};

ostream &operator<<(ostream &,HiddenMarkovModel &);

#endif
