/****************************************************************
 UnscaledBaumWelch.H
 bmajoros@duke.edu

 This is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
 ****************************************************************/

#ifndef INCL_ParallelBaumWelch_H
#define INCL_ParallelBaumWelch_H

#include "UnscaledForwardAlgorithm.H"
#include "UnscaledBackwardAlgorithm.H"
#include "HiddenMarkovModel.H"
#include <math.h>
#include <iostream>
#include <strstream>
#include <fstream>
#include <float.h>
#include "BOOM/Vector.H"
#include "BOOM/Array1D.H"
#include "BOOM/Sequence.H"
#include "BOOM/Progress.H"
#include "BOOM/Thread.H"
using namespace std;


struct BWThread : public Thread
{
    BWThread(int states,int numAlpha,HiddenMarkovModel &hmm,
             Vector<Sequence*> &trainingSet,int first, int last);
    virtual void f();
    void reviseExpectedEmitCounts(int sequenceLength,
                                  ForwardAlgorithm &,BackwardAlgorithm &,
                                  Sequence &);
    void reviseExpectedTransCounts(int sequenceLength,ForwardAlgorithm &,
                                   BackwardAlgorithm &,
                                   Sequence &);

    Array2D<double> A; // expected transition counts
    Array2D<double> E; // expected emission counts
    int first, last;
    HiddenMarkovModel &hmm;
    Vector<Sequence*> &trainingSet;
    Array1D<double> likelihoods;
    int states, alphabetSize;
};


class ParallelBaumWelch
{
public:
    ParallelBaumWelch(HiddenMarkovModel &,long maxIterations,
                      Vector<Sequence*> &trainingSet,int numThreads);
private:
    Alphabet &alphabet;
    Progress progress;
    Array1D<double> likelihoods;
    Array2D<double> A; // expected transition counts
    Array2D<double> E; // expected emission counts
    HiddenMarkovModel &hmm;
    long maxIterations;
    double logLikelihoodThreshold, sum;
    Vector<Sequence*> &trainingSet;
    int numHmmStates, numTrain, alphabetSize, numThreads;
    
    void mainAlgorithm();
};


#endif
