/**************************************************************
 log-likelihood.C
 bmajoros@duke.edu

 Ths is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
***************************************************************/

#include <string>
#include <iostream>
#include <fstream>
#include "BOOM/FastaReader.H"
#include "BOOM/Sequence.H"
#include "BOOM/DnaAlphabet.H"
#include "BOOM/CommandLine.H"
#include "HiddenMarkovModel.H"
using namespace std;

class Application
{
  DnaAlphabet alphabet;
public:
  int go(int argc,char *argv[]);
};


int main(int argc,char *argv[])
{
  try
    {
      Application app;
      return app.go(argc,argv);
    }
  catch(const char *msg)
    {
      cerr << msg << endl;
      return -1;
    }
  catch(const String &s)
    {
      cerr << s.c_str() << endl;
      return -1;
    }
  catch(...)
    {
      cerr << "unknown exception caught in main()" << endl;
      return -1;
    }
  return 0;
}



void usage()
{
    throw "log-likelihood <*.hmm> <*.fasta>";
}


int Application::go(int argc,char *argv[])
{
  // Process command line
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=2) usage();
  String hmmFile=cmd.arg(0);
  String fastaFile=cmd.arg(1);

  // Load HMM
  HiddenMarkovModel hmm(hmmFile,alphabet);

  // Process the FASTA file
  FastaReader reader(fastaFile);
  String defline, sequence;
  while(reader.nextSequence(defline,sequence))
    {
      Sequence seq(sequence,alphabet);
      cout<<hmm.getLogP(seq)<<endl;
    }

  return 0;
}


