/**************************************************************
random-HMM.C
bmajoros@duke.edu

This is OPEN SOURCE SOFTWARE governed by the ARTISTIC LICENSE.
***************************************************************/

#include <string>
#include <iostream>
#include <fstream>
#include "BOOM/Random.H"
#include "BOOM/DnaAlphabet.H"
#include "BOOM/CommandLine.H"
#include "HMMbuilder.H"
using namespace std;

class Application
{
  DnaAlphabet alphabet;
public:
  int go(int argc,char *argv[]);
};


int main(int argc,char *argv[])
{
  try
    {
      Application app;
      return app.go(argc,argv);
    }
  catch(const char *msg)
    {
      cerr << msg << endl;
      return -1;
    }
  catch(const String &s)
    {
      cerr << s.c_str() << endl;
      return -1;
    }
  catch(...)
    {
      cerr << "unknown exception caught in main()" << endl;
      return -1;
    }
  return 0;
}



void usage()
{
    throw "random-HMM <num-states> <connectivity[0-1]> <outfile>";
}



int Application::go(int argc,char *argv[])
{
  // Process command line
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=3) usage();
  int numStates=cmd.arg(0).asInt();
  double connectivity=cmd.arg(1).asDouble();
  String outfile=cmd.arg(2);
  randomize();

  // Generate random HMM
  HMMbuilder builder;
  HiddenMarkovModel *hmm=builder.randomHMM(numStates,connectivity,alphabet);

  // Save the results
  hmm->save(outfile);

  return 0;
}



