
// viterbi-trainer.C

#include <iostream>
#include "BOOM/CommandLine.H"
#include "BOOM/Vector.H"
#include "BOOM/Sequence.H"
#include "BOOM/DnaAlphabet.H"
#include "BOOM/Array2D.H"
#include "BOOM/FastaReader.H"
#include "FastViterbi.H"
using namespace std;


class Application
{
  Vector<Sequence*> trainingSet;
  Vector<String> trainingSequences;
  DnaAlphabet alphabet;

  void load(String filename);
public:
  Application();
  int main(int argc,char *argv[]);
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
  {
    // Process command line
    CommandLine cmd(argc,argv,"");
    if(cmd.numArgs()!=3)
      throw string("viterbi-trainer <template.hmm> <training-features.fasta> <outfile>");
    String inHmmFile=cmd.arg(0);
    String inTrainFile=cmd.arg(1);
    String outHmmFile=cmd.arg(2);

    // Load HMM and perform some initialization
    HiddenMarkovModel oldHmm(inHmmFile,alphabet);
    FastViterbi viterbi(oldHmm);

    // Load training set
    load(inTrainFile);
    int numTrain=trainingSet.size();

    // Iterate through all training sequences
    for(int i=0 ; i<numTrain ; ++i)
      {
	Sequence &seq=*trainingSet[i];
	const String &seqStr=trainingSequences[i];
	double pathScore;
	vector<int> &path=*viterbi.getPath(seq,pathScore);
	int len=path.size();
	for(int j=0 ; j<len ; ++j)
	  {
	    int begin=j-newOrder;
	    if(begin<0) begin=0;
	    int ngramLen=j-begin+1;
	    String ngram=seqStr.substr(begin,ngramLen);
	    cout<<ngram<<" "<<path[j]<<endl;
	  }
      }
    
    return 0;
  }



void Application::load(String filename)
{
  FastaReader reader(filename);
  String defline, sequence;
  while(reader.nextSequence(defline,sequence))
    {
      trainingSequences.push_back(sequence);
      Sequence *seq=new Sequence(sequence,alphabet);
      trainingSet.push_back(seq);
    }
}

